package cz.fidentis.web.batchprocessing.registration;

import cz.fidentis.analyst.Logger;
import cz.fidentis.analyst.data.face.HumanFace;
import cz.fidentis.analyst.data.face.HumanFaceFactory;
import cz.fidentis.analyst.data.mesh.MeshIO;
import cz.fidentis.analyst.data.mesh.MeshModel;
import cz.fidentis.analyst.engines.avgmesh.AvgMeshConfig;
import cz.fidentis.analyst.engines.avgmesh.AvgMeshVisitor;
import cz.fidentis.analyst.engines.distance.MeshDistanceConfig;
import cz.fidentis.analyst.engines.distance.MeshDistanceServices;
import cz.fidentis.analyst.engines.distance.measurement.DistanceRecord;
import cz.fidentis.analyst.engines.distance.measurement.MeshDistances;
import cz.fidentis.analyst.engines.face.FaceRegistrationServices;
import cz.fidentis.analyst.engines.face.FaceStateServices;
import cz.fidentis.analyst.engines.icp.IcpConfig;
import cz.fidentis.analyst.engines.sampling.PointSamplingConfig;
import cz.fidentis.web.async.WebSocketProgressHandler;
import cz.fidentis.web.batchprocessing.registration.dto.BatchRegistrationTaskDto;
import cz.fidentis.web.batchprocessing.registration.enums.BatchAveragingMethod;
import cz.fidentis.web.batchprocessing.registration.enums.BatchRegistrationMethod;
import cz.fidentis.web.fileupload.FileUpload;
import cz.fidentis.web.fileupload.FileUploadRepo;
import cz.fidentis.web.fileupload.FileUploadService;
import cz.fidentis.web.humanface.HumanFaceEntity;
import cz.fidentis.web.humanface.dto.HumanFaceDto;
import cz.fidentis.web.humanface.service.HumanFaceService;
import cz.fidentis.web.task.Task;
import cz.fidentis.web.task.TaskRepo;
import jakarta.persistence.EntityNotFoundException;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.SerializationUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.List;
import java.util.Optional;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.CompletionException;

/**
 * @author Patrik Tomov
 * @since  06.03.2024
 */
@Service
@RequiredArgsConstructor
@Slf4j
public class BatchRegistrationServiceImpl implements BatchRegistrationService {

    private static final int ICP_ITERATIONS = 100;
    private static final double ICP_ERROR = 0.05;
    private static final int ICP_AUTO_CROP_SINCE = 1;

    private final HumanFaceService humanFaceService;
    private final FileUploadService fileUploadService;
    private final FileUploadRepo fileUploadRepo;
    private final TaskRepo taskRepo;

    private final WebSocketProgressHandler webSocketProgressHandler;

    @Override
    @Transactional
    public HumanFaceDto execute(BatchRegistrationTaskDto dto) {
        HumanFace selectedFace = setHumanFaceEntitiesAndGetSelectedFace(dto);

        HumanFace newAvgFace = executeRegistration(dto, selectedFace);

        return createOrUpdateAverageFaceFile(dto, newAvgFace);
    }

    @Override
    public byte[] exportAverageFace(Long humanFaceEntityId) {
        HumanFaceEntity humanFaceEntity = humanFaceService.getHumanFaceEntityById(humanFaceEntityId);

        HumanFace avgFace = SerializationUtils.deserialize(humanFaceEntity.getHumanFaceDump());
        try {
            File file = createTempFileForAverageFace();
            MeshIO.exportMeshModel(avgFace.getMeshModel(), file);
            return Files.readAllBytes(file.toPath());
        } catch (IOException e) {
            throw new RuntimeException("Failed to create or read the file", e);
        }
    }

    private HumanFace executeRegistration(BatchRegistrationTaskDto dto, HumanFace selectedFace) {
        HumanFace newAvgFace;
        double prevPSM = 0;
        int iter = 1;
        while (true) {
            newAvgFace = averageAndRegisterIteration(dto);
            if (newAvgFace == null) {
                return null;
            }
            if (iter >= dto.getAveragingTaskDto().getMaxRegistrationIterations()) {
                break;
            }

            double newPSM = procrustesSurfaceMetric(selectedFace, newAvgFace);
            double psmDiff = Math.abs(prevPSM - newPSM);

            Logger.print("Iter. " + iter + " (PSM, |diff|): " + newPSM + ", " + psmDiff);

            if (psmDiff < dto.getAveragingTaskDto().getStabilizationThreshold()) {
                break;
            }

            selectedFace = newAvgFace;
            prevPSM = newPSM;
            iter++;
        }
        return newAvgFace;
    }

    private HumanFace setHumanFaceEntitiesAndGetSelectedFace(BatchRegistrationTaskDto dto) {
        List<HumanFaceEntity> faceEntities = humanFaceService.getHumanFaceEntitiesOfTask(dto.getTaskId());
        dto.setHumanFaceEntities(faceEntities);

        return humanFaceService.getHumanFaceEntityByFileUploadIdAndTaskId(dto.getSelectedFile().getId(), dto.getTaskId())
                .getHumanFace();
    }

    private HumanFaceDto createOrUpdateAverageFaceFile(BatchRegistrationTaskDto dto, HumanFace newAvgFace) {
        FileUpload averageFaceFile = fileUploadService.getAverageFaceFileOfTask(dto.getTaskId());
        FileUpload fileUpload = fileUploadService.createOrUpdateAverageFaceFile(dto.getProjectId(), dto.getTaskId(), averageFaceFile, newAvgFace);
        return humanFaceService.getHumanFaceDtoByFileUploadIdAndTaskId(fileUpload.getId(), dto.getTaskId());
    }

    private static double procrustesSurfaceMetric(HumanFace f1, HumanFace f2) {
        double distanceF2toF1 = computeDistance(f1, f2);
        double distanceF1toF2 = computeDistance(f2, f1);

        return Math.sqrt(distanceF2toF1 + distanceF1toF2);
    }

    private static double computeDistance(HumanFace f1, HumanFace f2) {
        FaceStateServices.updateKdTree(f1, FaceStateServices.Mode.COMPUTE_IF_ABSENT);

        MeshDistances distances = MeshDistanceServices.measure(
                f2.getMeshModel(),
                new MeshDistanceConfig(
                        MeshDistanceConfig.Method.POINT_TO_TRIANGLE_NEAREST_NEIGHBORS,
                        f1.getKdTree(),
                        false,
                        true
                )
        );

        List<Double> distancesF2toF1 = distances.stream()
                .mapToDouble(DistanceRecord::getDistance)
                .filter(Double::isFinite)
                .boxed()
                .toList();

        double distance = distancesF2toF1.stream().mapToDouble(d -> d*d).sum();
        distance *= 1.0 / (2.0 * distancesF2toF1.size());
        return distance;
    }

    private HumanFace averageAndRegisterIteration(BatchRegistrationTaskDto dto) {
        try {
            return doInBackground(dto).join();
        } catch (CompletionException ex) {
            Logger.print(ex.getCause().toString());
            return null;
        }
    }

    @Async
    protected CompletableFuture<HumanFace> doInBackground(BatchRegistrationTaskDto dto) {
        BatchRegistrationMethod regStrategy = dto.getRegistrationTaskDto().getRegistrationMethod();
        BatchAveragingMethod avgStrategy = dto.getAveragingTaskDto().getAveragingMethod();
        boolean doRegistration = regStrategy.equals(BatchRegistrationMethod.FEATURE_POINTS)
                || regStrategy.equals(BatchRegistrationMethod.MESH_BASED_ICP);
        boolean computeAvgFace = avgStrategy.equals(BatchAveragingMethod.NEAREST_NEIGHBOURS)
                || avgStrategy.equals(BatchAveragingMethod.RAY_CASTING);

        if (!(doRegistration || computeAvgFace)) {
            return CompletableFuture.completedFuture(null);
        }

        // Get initial (template) face. Either the average face is chosen if exists (we want to improve previous iteration),
        // or the selected face is chosen (the first iteration).
        // We don't need to reload the initFace periodically for two reasons:
        //   - It is never dumped from memory to disk because we use MRU
        //   - Even if dumped, the face keeps in the memory until we hold the pointer to it
        HumanFace initFace = getTemplateFace(dto);

        // Iterate through all faces to register them and/or compute new average face.
        // Process all faces, including the selected one, because the template can vary.
        AvgMeshVisitor avgFaceConstructor = null;
        int numFaces = dto.getFileIds().size();
        for (int i = 0; i < numFaces; i++) {
            HumanFace superimposedFace = getHumanFaceAtIndexFromArray(i, dto); // Load face to be processed and possibly transformed

            if (doRegistration) { // superimpose superimposedFace to initFace
                register(regStrategy, initFace, superimposedFace, dto);
            }

            if (computeAvgFace) { // update the average face
                avgFaceConstructor = (avgStrategy.equals(BatchAveragingMethod.NEAREST_NEIGHBOURS))
                        ? computeAvgFaceNN(initFace, superimposedFace, avgFaceConstructor, doRegistration)
                        : computeAvgFaceRT(initFace, superimposedFace, avgFaceConstructor, doRegistration);
            }
            int progressValue = (int) Math.round(100.0 * (i + 1.0) / numFaces);
            webSocketProgressHandler.updateProgress(progressValue);
        }
        HumanFace avgFace = null;
        if (avgFaceConstructor != null) { // create new human face from the average face geometry
            try {
                avgFace = createAvgFace(avgFaceConstructor.getAveragedMeshModel());
            } catch (IOException ex) {
                Logger.print(ex.toString());
            }
        }
        return CompletableFuture.completedFuture(avgFace);
    }

    private HumanFace getTemplateFace(BatchRegistrationTaskDto dto) {
        Long selectedFileId = Optional.ofNullable(fileUploadService.getAverageFaceFileOfTask(dto.getTaskId()))
                .map(FileUpload::getId)
                .orElse(dto.getSelectedFile().getId());

        HumanFaceEntity humanFaceEntity = humanFaceService.getHumanFaceEntityByFileUploadIdAndTaskId(selectedFileId, dto.getTaskId());
        return humanFaceEntity.getHumanFace();
    }


    private File createTempFileForAverageFace() throws IOException {
        File file = File.createTempFile("AverageFace", ".obj");
        file.deleteOnExit();
        return file;
    }

    protected HumanFace createAvgFace(MeshModel avgMesh) throws IOException {
        File tempFile = File.createTempFile(this.getClass().getSimpleName(), ".obj");
        tempFile.deleteOnExit();

        HumanFace newAvgFace = HumanFaceFactory.create(avgMesh, tempFile.getCanonicalPath());
        try {
            MeshIO.exportMeshModel(newAvgFace.getMeshModel(), tempFile);
        } catch (IOException ex) {
            Logger.print(ex.toString());
        }

        return newAvgFace;
    }

    protected AvgMeshVisitor computeAvgFaceNN(HumanFace initFace, HumanFace superimposedFace, AvgMeshVisitor avgFaceConstructor, boolean doRegistration) {
        // If the face was moved by registration, then the spatial ordering structure was removed => create it
        // If no transformation was made, then you can use old structure, if exists
        FaceStateServices.updateKdTree(
                superimposedFace,
                doRegistration ? FaceStateServices.Mode.COMPUTE_ALWAYS : FaceStateServices.Mode.COMPUTE_IF_ABSENT
        );

        AvgMeshVisitor ret = (avgFaceConstructor == null)
                ? (new AvgMeshConfig(initFace.getMeshModel(), null)).getNearestNeighborsVisitor()
                : avgFaceConstructor;
        superimposedFace.getKdTree().accept(ret);

        return ret;
    }

    protected AvgMeshVisitor computeAvgFaceRT(HumanFace initFace, HumanFace superimposedFace, AvgMeshVisitor avgFaceConstructor, boolean doRegistration) {
        // If the face was moved by registration, then the spatial ordering structure was removed => create it
        // If no transformation was made, then you can use old structure, if exists
        FaceStateServices.updateOctree(
                superimposedFace,
                doRegistration ? FaceStateServices.Mode.COMPUTE_ALWAYS : FaceStateServices.Mode.COMPUTE_IF_ABSENT
        );

        AvgMeshVisitor ret = (avgFaceConstructor == null)
                ? (new AvgMeshConfig(initFace.getMeshModel(), null)).getRayCastingVisitor()
                : avgFaceConstructor;
        superimposedFace.getOctree().accept(ret);

        return ret;
    }

    protected void register(BatchRegistrationMethod regStrategy, HumanFace initFace, HumanFace superimposedFace, BatchRegistrationTaskDto dto) {
        switch (regStrategy) {
            case MESH_BASED_ICP -> { // ICP registration - the face is transformed!
                int subsampling = dto.getRegistrationTaskDto().getRegistrationSubsampling();
                PointSamplingConfig sampling = (subsampling == 0)
                        ? new PointSamplingConfig(PointSamplingConfig.Method.NO_SAMPLING, subsampling)
                        : new PointSamplingConfig(PointSamplingConfig.Method.RANDOM, subsampling);
                FaceRegistrationServices.alignMeshes(
                        superimposedFace, // is transformed
                        new IcpConfig(
                                initFace.getMeshModel(),
                                ICP_ITERATIONS,
                                dto.getRegistrationTaskDto().isScaleFacesDuringRegistration(),
                                ICP_ERROR,
                                sampling,
                                ICP_AUTO_CROP_SINCE));
            }
            case FEATURE_POINTS -> // GPA registration - the face is transformed!
                    FaceRegistrationServices.alignFeaturePoints(
                            initFace,
                            superimposedFace, // is transformed
                            dto.getRegistrationTaskDto().isScaleFacesDuringRegistration());
            default -> throw new IllegalStateException("Unexpected value: " + regStrategy);
        }
    }

    private HumanFace getHumanFaceAtIndexFromArray(int faceIndex, BatchRegistrationTaskDto dto) {
        Long fileId = dto.getFileIds().get(faceIndex);
        try {
            HumanFaceEntity humanFaceEntity = humanFaceService.getHumanFaceEntityByFileUploadIdAndTaskId(fileId, dto.getTaskId());
            return humanFaceEntity.getHumanFace();
        } catch (EntityNotFoundException e) {
            return createAndFetchHumanFace(fileId, dto.getTaskId());
        }
    }

    private HumanFace createAndFetchHumanFace(Long fileId, Long taskId) {
        FileUpload fileUpload = fileUploadRepo.getReferenceById(fileId);
        Task task = taskRepo.getReferenceById(taskId);
        HumanFaceEntity newHumanFaceEntity = humanFaceService.createHumanFace(fileUpload, task);
        newHumanFaceEntity.setHumanFace(SerializationUtils.deserialize(newHumanFaceEntity.getHumanFaceDump()));
        log.info("Human face not found, creating new one");
        return newHumanFaceEntity.getHumanFace();
    }
}
