package cz.fidentis.web.task;

import cz.fidentis.web.exceptions.exceptions.EntityNotFoundException;
import cz.fidentis.web.fileupload.FileUpload;
import cz.fidentis.web.fileupload.FileUploadRepo;
import cz.fidentis.web.humanface.HumanFaceEntity;
import cz.fidentis.web.humanface.service.HumanFaceService;
import cz.fidentis.web.project.Project;
import cz.fidentis.web.project.ProjectRepo;
import cz.fidentis.web.task.dto.CreateTaskDto;
import cz.fidentis.web.task.dto.TaskDto;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

import static cz.fidentis.web.cache.CacheConfig.TASK_FACES;

/**
 * @author Petr Hendrych
 * @since 09.02.2023
 */
@Service
@RequiredArgsConstructor
@Transactional
@Slf4j
public class TaskServiceImpl implements TaskService {

    private final HumanFaceService humanFaceService;
    private final FileUploadRepo fileUploadRepo;
    private final ProjectRepo projectsRepo;
    private final TaskRepo taskRepo;

    @Override
    public List<TaskDto> getProjectTasks(Long projectId) {
        return taskRepo.findAllByProjectId(projectId);
    }

    @CacheEvict(value = TASK_FACES, key = "@cacheService.tokenId")
    @Override
    public TaskDto createTask(CreateTaskDto dto) {
        Project project = projectsRepo.findById(dto.getProjectId())
                .orElseThrow(() -> new EntityNotFoundException("Project with id: %s does not exist".formatted(dto.getProjectId())));

        Task task = new Task();
        project.addTask(task);
        task.setTaskType(TaskType.getTaskTypeForNumberOfFaces(dto.getFileUploadIds().size()));

        Task savedTask = taskRepo.save(task);


        List<FileUpload> fileUploads = fileUploadRepo.findAllById(dto.getFileUploadIds());

        if (task.getTaskType().equals(TaskType.BATCH_PROCESSING)) {
            fileUploads.forEach(file -> file.addTask(savedTask));
            fileUploadRepo.findById(dto.getFileUploadIds().get(0)).ifPresent(fileUpload -> {
                HumanFaceEntity humanFace = humanFaceService.createHumanFace(fileUpload, savedTask);
                savedTask.addHumanFace(humanFace);
            });
        } else {
            List<HumanFaceEntity> humanFaces = createHumanFaces(fileUploads, dto.getFileUploadIds(), savedTask);
            savedTask.addHumanFaces(humanFaces);
        }

        task.setName(getName(task, dto.getFileUploadIds()));

        return taskRepo.findTaskById(savedTask.getId())
                .orElseThrow(() -> new EntityNotFoundException("Task with id %s does not exist".formatted(savedTask.getId())));
    }

    private String getName(Task task, List<Long> ids) {
        switch (task.getTaskType()) {
            case SINGLE_FACE_ANALYSIS -> {
                return task.getFileUploads().stream()
                        .findAny().orElseThrow().getFaceInfo().getGeometryFileName().split(".obj")[0];
            }
            case PAIR_COMPARISON -> {
                var primaryFaceInfo = task.getFileUploads().stream()
                        .filter(f -> f.getId().equals(ids.get(0))).findAny().orElseThrow().getFaceInfo();
                var secondaryFaceInfo = task.getFileUploads().stream()
                        .filter(f -> f.getId().equals(ids.get(1))).findAny().orElseThrow().getFaceInfo();
                return primaryFaceInfo.getGeometryFileName().split(".obj")[0] + ":" +
                        secondaryFaceInfo.getGeometryFileName().split(".obj")[0];
            }
            case BATCH_PROCESSING -> {
                return "Task " + task.getId() + ": " + ids.size() + " faces";
            }
            default -> throw new IllegalStateException("Task must have one of three TaskTypes");
        }
    }

    @Override
    public void deleteTask(Long id) {
        log.info("Deleting task with id: {}", id);
        Task task = taskRepo.findById(id)
                .orElseThrow(() -> new EntityNotFoundException("Task with id: %s does not exist".formatted(id)));
        task.getFileUploads().forEach(file -> file.getTasks().remove(task));
        taskRepo.delete(task);
    }

    @Override
    public TaskDto getTaskDtoById(Long id) {
        return taskRepo.findTaskById(id)
                .orElseThrow(() -> new EntityNotFoundException("Task with id: %s does not exist".formatted(id)));
    }

    @Override
    public Task getTaskById(Long id) {
        return taskRepo.findById(id)
                .orElseThrow(() -> new EntityNotFoundException("Task with id: %s does not exist".formatted(id)));
    }

    @Override
    public TaskDto updateTaskName(String name, Long id) {
        Task task = taskRepo.findById(id)
                .orElseThrow(() -> new EntityNotFoundException("Task with id: %s does not exist".formatted(id)));

        task.setName(name);
        taskRepo.save(task);

        return taskRepo.findTaskById(id)
                .orElse(null);
    }

    private List<HumanFaceEntity> createHumanFaces(List<FileUpload> fileUploads, List<Long> ids, Task task) {
        return fileUploads.stream()
                .map(fileUpload -> {
                    HumanFaceEntity humanFace = humanFaceService.createHumanFace(fileUpload, task);
                    fileUpload.addTask(task);
                    if (task.getTaskType().equals(TaskType.PAIR_COMPARISON) && fileUpload.getId().equals(ids.get(0))) {
                        humanFace.setIsPrimaryFace(true);
                    }
                    return humanFace;
                })
                .toList();
    }
}
