import {
	ChangeEvent,
	SyntheticEvent,
	useContext,
	useEffect,
	useState
} from 'react';
import { SelectChangeEvent } from '@mui/material';

import { TaskContext } from '../providers/TaskContextProvider';
import { ProjectsContext } from '../providers/ProjectContextProvider';
import { BatchProcessingTaskContext } from '../providers/BatchProcessingTaskContextProvider';
import { SceneContext } from '../providers/SceneContextProvider';
import { FileUpload } from '../types/FileTypes';
import { HumanFace } from '../types/HumanFaceTypes';
import { BatchProcessingTaskService } from '../services/BatchProcessingTaskService';
import { LoadingOperation } from '../types/AnalyticalTaskTypes';
import { DownloadUtils } from '../utils/DownloadUtils';
import {
	AveragingMethod,
	BatchRegistrationTaskDto,
	RegistrationMethod
} from '../types/BatchProcessingTypes';

const AVERAGE_FACE_FILE_NAME = 'Average Face';
let socket: WebSocket | null = null;

const useBatchTaskManagement = () => {
	const { task } = useContext(TaskContext);
	const { selectedProjectId } = useContext(ProjectsContext);
	const { taskFiles, isLoadingInfo, getTaskFiles, updateLoadingInfo } =
		useContext(BatchProcessingTaskContext);
	const { setFace } = useContext(SceneContext);

	const [selectedFile, setSelectedFile] = useState<FileUpload | null>(null);
	const [averageFace, setAverageFace] = useState<HumanFace | null>(null);
	const [iteration, setIteration] = useState(1);
	const [progress, setProgress] = useState(0);
	const [showProgressBar, setShowProgressBar] = useState(false);

	const [batchRegistrationTaskDto, setBatchRegistrationTaskDto] =
		useState<BatchRegistrationTaskDto>({
			taskId: task?.id ?? -1,
			projectId: selectedProjectId ?? -1,
			selectedFile: taskFiles[0],
			fileIds: [],
			averagingTaskDto: {
				taskId: task?.id ?? -1,
				averagingMethod: AveragingMethod.NearestNeighbours,
				stabilizationThreshold: 0,
				maxRegistrationIterations: 3
			},
			registrationTaskDto: {
				taskId: task?.id ?? -1,
				registrationMethod: RegistrationMethod.MeshBasedICP,
				registrationSubsampling: 1000,
				scaleFacesDuringRegistration: false
			}
		});

	useEffect(() => {
		getTaskFiles();
	}, []);

	useEffect(() => {
		if (taskFiles.length > 0) {
			setDefaultOrAverageFace();
			setBatchRegistrationTaskDto({
				...batchRegistrationTaskDto,
				fileIds: taskFiles.map(file => file.id)
			});
		}
	}, [taskFiles]);

	useEffect(() => {
		setBatchRegistrationTaskDto({
			...batchRegistrationTaskDto,
			selectedFile
		});
	}, [selectedFile]);

	const setDefaultOrAverageFace = () => {
		const avgFace = taskFiles.find(
			file => file.name === AVERAGE_FACE_FILE_NAME
		);
		if (avgFace) {
			setSelectedFile(avgFace);
		} else {
			setSelectedFile(taskFiles[0]);
		}
	};

	const exportAverageFace = async () => {
		if (!averageFace) return;
		const blob = await BatchProcessingTaskService.exportAverageFace(
			averageFace.id
		);
		const fileName = 'AverageFace.obj';

		DownloadUtils.downloadFile(blob, fileName);
	};

	const closeWebSocket = () => {
		if (socket) {
			socket.close();
			console.log('WebSocket connection closed');
		}
	};

	const handleAveragingOptionsChange = (which: string, value: number) => {
		if (which === 'iterations') {
			setBatchRegistrationTaskDto({
				...batchRegistrationTaskDto,
				averagingTaskDto: {
					...batchRegistrationTaskDto.averagingTaskDto,
					maxRegistrationIterations: value
				}
			});
		} else {
			setBatchRegistrationTaskDto({
				...batchRegistrationTaskDto,
				averagingTaskDto: {
					...batchRegistrationTaskDto.averagingTaskDto,
					stabilizationThreshold: value
				}
			});
		}
	};

	const handleRegistrationOptionChange = (value: number) => {
		setBatchRegistrationTaskDto({
			...batchRegistrationTaskDto,
			registrationTaskDto: {
				...batchRegistrationTaskDto.registrationTaskDto,
				registrationSubsampling: value
			}
		});
	};

	const handleCheckboxChange = (e: ChangeEvent<HTMLInputElement>) => {
		setBatchRegistrationTaskDto({
			...batchRegistrationTaskDto,
			registrationTaskDto: {
				...batchRegistrationTaskDto.registrationTaskDto,
				scaleFacesDuringRegistration: e.target.checked
			}
		});
	};

	const handleChangeTemplate = (
		e: ChangeEvent<HTMLInputElement | HTMLTextAreaElement> | SelectChangeEvent
	) => {
		const fileName = e.target.value;
		const newSelectedFile = taskFiles.find(
			file => `${file.name}.${file.extension}` === fileName
		);
		if (newSelectedFile) {
			setSelectedFile(newSelectedFile);
			setFace(null, newSelectedFile.id, task?.id);
		}
	};

	const simulateProgress = () => {
		setShowProgressBar(true);
		socket = new WebSocket('ws://localhost:8080/progress');

		socket.addEventListener('open', () => {
			if (socket) {
				socket.send('Connection established');
			}
		});

		socket.addEventListener('message', event => {
			console.log('Message from server ', event.data);
			setProgress(Number(event.data));
			if (Number(event.data) === 100) {
				setIteration(currentIteration => currentIteration + 1);
			}
		});
	};

	const handleSubmit = async (e: SyntheticEvent) => {
		e.preventDefault();
		simulateProgress();
		updateLoadingInfo(true, LoadingOperation.Registration);
		try {
			const humanFace = await BatchProcessingTaskService.executeRegistration(
				batchRegistrationTaskDto
			);
			setFace(humanFace, null, task?.id);
			setAverageFace(humanFace);
			getTaskFiles();
		} finally {
			closeWebSocket();
			setShowProgressBar(false);
			setIteration(1);
			setProgress(0);
			updateLoadingInfo(false);
		}
	};

	return {
		batchRegistrationTaskDto,
		taskFiles,
		selectedFile,
		averageFace,
		iteration,
		progress,
		showProgressBar,
		setBatchRegistrationTaskDto,
		isLoadingInfo,
		exportAverageFace,
		handleAveragingOptionsChange,
		handleRegistrationOptionChange,
		handleCheckboxChange,
		handleChangeTemplate,
		handleSubmit
	};
};

export default useBatchTaskManagement;
