import { createContext, FC, useContext, useMemo, useState } from 'react';

import { LoadingOperation } from '../types/AnalyticalTaskTypes';
import { FileUpload } from '../types/FileTypes';
import { FileUploadService } from '../services/FileUploadService';

import { TaskContext } from './TaskContextProvider';

type LoadingInfo = {
	isLoading: boolean;
	operation: LoadingOperation;
};

type Props = {
	children: JSX.Element;
};

type BatchProcessingTaskContextValue = {
	taskFiles: FileUpload[];
	getTaskFiles: () => void;
	updateLoadingInfo: (isLoading: boolean, operation?: LoadingOperation) => void;
	isLoadingInfo: (Operation?: LoadingOperation, isEqual?: boolean) => boolean;
};

export const BatchProcessingTaskContextProvider: FC<Props> = ({ children }) => {
	const { task, setIsBatchProcessingTaskLoading } = useContext(TaskContext);
	const [taskFiles, setTaskFiles] = useState<FileUpload[]>([]);
	const [loadingInfo, setLoadingInfo] = useState<LoadingInfo>({
		isLoading: false,
		operation: LoadingOperation.None
	});

	const getTaskFiles = async () => {
		updateLoadingInfo(true, LoadingOperation.Files);
		try {
			if (!task?.id) {
				return null;
			}
			const res = await FileUploadService.getFilesOfTask(task.id);
			setTaskFiles(res);
		} finally {
			updateLoadingInfo(false);
		}
	};

	const updateLoadingInfo = (
		isLoading: boolean,
		operation?: LoadingOperation
	) => {
		setIsBatchProcessingTaskLoading(isLoading);

		setLoadingInfo({
			isLoading,
			operation: operation ?? LoadingOperation.None
		});
	};

	const isLoadingInfo = (Operation?: LoadingOperation, isEqual?: boolean) => {
		const isLoading = loadingInfo.isLoading;

		if (Operation) {
			if (isEqual) {
				return isLoading && loadingInfo.operation === Operation;
			}

			return isLoading && loadingInfo.operation !== Operation;
		}

		return isLoading;
	};

	const value = useMemo<BatchProcessingTaskContextValue>(
		() => ({
			taskFiles,
			getTaskFiles,
			updateLoadingInfo,
			isLoadingInfo
		}),
		[taskFiles, getTaskFiles, updateLoadingInfo, isLoadingInfo]
	);

	return (
		<BatchProcessingTaskContext.Provider value={value}>
			{children}
		</BatchProcessingTaskContext.Provider>
	);
};

export const BatchProcessingTaskContext =
	createContext<BatchProcessingTaskContextValue>(undefined as never);
